%%  MIMIC CONNECTION in Matlab
% Machine Learning in Critical Care.
% Lesson #2: Visualizing the data 
% Mohamad M. Ghassemi
clear all;

%% STEP 0: A quick note about executing Matlab
%If you want to run a section of this code, 
% 1. Go to the EDITOR tab,
% 2. Click the section you want to run, 
% 3. Click "Run Section" icon (or Ctrl+Enter) 


%% STEP 1: Let's load the data.
% you can also do this in the GUI by double clicking the '.mat' file
% on the left hand side panel. Or you can run...
load('MLCCData.mat');

%% STEP 2: Let's Unpack the Data
for i = 1:size(data,2)
    name = header{i};
    eval([name '= data(:,i);']); 
end

%LOOK AT YOUR WORKSPACE NOW! ----->

%% STEP 3: Let's Visualize some features, starting with Age
%Visualize the age of all the patients.
hist(Age,20); xlabel('Value'); ylabel('Number'); title('Age - everyone');

% What do we observe? There seems to be a large group of outliers, but this
% is explained in the documentation online as patients over the age of 89.
% See here for more details:
% https://mimic.physionet.org/mimictables/patients/

%% STEP 4: Replace the patients with artificial age values, and plot again! 

% There are some patients with artificial ages, let's replace those ages
%with the median that is reported in the documentation.
age_shifted = find(Age > 200);
Age(age_shifted) = 91.4;

%Visualize the age of all the patients.
hist(Age,20); xlabel('Value'); ylabel('Number'); title('Age (corrected) - everyone');


%% STEP 5: Compare Age of patients that survived, versus non-survived
died = find(OUTCOME == 1);
survived = find(OUTCOME == 0);

%Compare the histogram
figure;
compare_hists(Age(died),Age(survived),20,0)
xlabel('Age'); ylabel('Number'); title('Age Histograms');
legend('died','survived')

%Do the histograms with Probabilities 
figure;
compare_hists(Age(died),Age(survived),20,1)
xlabel('Age'); ylabel('Probability'); title('Age Histograms');
legend('died','survived')


%% STEP 6: The Ageist Algorithm

%The Ageist Algorithm
figure;
prediction = (Age < 30 | Age > 75 );

%performance matrix
plotconfusion(OUTCOME',prediction')


%% STEP 7: More Features

%Heart Rate
figure;
compare_hists(HeartRateMax(died),HeartRateMax(survived),30,1)
xlabel('HeartRateMax'); ylabel('Probability');
legend('died','survived')

figure;
compare_hists(HeartRateMin(died),HeartRateMin(survived),30,1)
xlabel('HeartRatemin'); ylabel('Probability');
legend('died','survived')

%White Blood Cell Count
figure;
compare_hists(WBCmax(died),WBCmax(survived),1000,1)
xlabel('WBCMax'); ylabel('Probability');
legend('died','survived')

figure;
compare_hists(WBCmin(died),WBCmin(survived),1000,1)
xlabel('WBCMin'); ylabel('Probability');
legend('died','survived')

%% STEP 8: The Better Black Box

%The HR,WBC Algorithm
figure;
prediction = ((WBCmax > 15 & WBCmin > 15) & ... 
              HeartRateMin > 100);
plotconfusion(OUTCOME',prediction')
classperf(OUTCOME',prediction)

%% STEP 9: The Optimist's Algorithm

%The optimist's Algorithm
figure;
prediction = zeros(1,length(Age));
plotconfusion(OUTCOME',prediction)
classperf(OUTCOME',prediction)

%% STEP 10: Re-evaluate the Optimist
figure;
prediction = zeros(1,length(Age));
[X,Y,T,AUC] = perfcurve(OUTCOME',prediction',1);
plot(X,Y); title(['Area Under ROC = ' num2str(AUC)]); 
xlabel('1 - Specificity'); ylabel('Sensitivity')

%% STEP 10: Re-evaluate Our Approach

prediction = double((WBCmax > 15 & WBCmin > 15) & ... 
              HeartRateMin > 100);
[X,Y,T,AUC] = perfcurve(OUTCOME',prediction',1);
plot(X,Y); title(['Area Under ROC = ' num2str(AUC)]); 
xlabel('1 - Specificity'); ylabel('Sensitivity')


%% STEP 11: The Democracy Algorithm

figure;
prediction =  (double(Age < 30) + ...
              double(Age > 75) + ... 
              double(WBCmax > 15) + ...
              double(WBCmin > 15) + ...
              double(HeartRateMax > 125) + ...
              double(HeartRateMin > 100) + ...
              double(HeartRateMin < 40))/7;
     
[X,Y,T,AUC] = perfcurve(OUTCOME',double(prediction)',1);
plot(X,Y); title(['Area Under ROC = ' num2str(AUC)]); 
xlabel('1 - Specificity'); ylabel('Sensitivity')


%% STEP 12: The Aristocracy Algorithm
figure;
prediction =  (4 * double(Age < 30) + ...
              1 * double(Age > 75) + ... 
              1 * double(WBCmax > 15) + ...
              1 * double(WBCmin > 15) + ...
              1 * double(HeartRateMax > 125) + ...
              3 * double(HeartRateMin > 100) + ...
              3 * double(HeartRateMin < 40))/14;

[X,Y,T,AUC] = perfcurve(OUTCOME',double(prediction)',1);
plot(X,Y); title(['Area Under ROC = ' num2str(AUC)]); 
xlabel('1 - Specificity'); ylabel('Sensitivity')




%% STEP 13: Let's Try a Specific Subject [290851]

%get the row for this patient
patient_index = find(ICUSTAYID == 290851)

%make the prediction
prediction =  (4 * double(Age(patient_index) < 30) + ...
              1 * double(Age(patient_index) > 75) + ... 
              1 * double(WBCmax(patient_index) > 15) + ...
              1 * double(WBCmin(patient_index) > 15) + ...
              1 * double(HeartRateMax(patient_index) > 125) + ...
              3 * double(HeartRateMin(patient_index) > 100) + ...
              3 * double(HeartRateMin(patient_index) < 40))/14
          
          
%What's going on here? Seems we have a NaN      
HeartRateMax(patient_index)
HeartRateMin(patient_index)

%Let's replace it
HeartRateMax(patient_index) = nanmedian(HeartRateMax);
HeartRateMin(patient_index) = nanmedian(HeartRateMin);

prediction =  (4 * double(Age(patient_index) < 30) + ...
              1 * double(Age(patient_index) > 75) + ... 
              1 * double(WBCmax(patient_index) > 15) + ...
              1 * double(WBCmin(patient_index) > 15) + ...
              1 * double(HeartRateMax(patient_index) > 125) + ...
              3 * double(HeartRateMin(patient_index) > 100) + ...
              3 * double(HeartRateMin(patient_index) < 40))/14
          
%Let's check the results:
OUTCOME(patient_index)
          
         




